!---------------------------------------------------------------------------------------
! FLEXINVERT : averageobs
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Average the observations to the model timestep 
!! Note: observations from moving receptors (ships and aircraft) are not averaged
!---------------------------------------------------------------------------------------

subroutine averageobs(knest,kout,bkgnd,knest_obs,kout_obs,bkgnd_obs,allobs)

  use mod_var
  use mod_dates

  implicit none

  character(len=7) :: recid
  integer :: ierr,nr,i,j,cnt,cntobs,ind
  integer :: rnum,timestep,yyyymmdd,hh,hloc
  real :: rlon,rlat,ralt,con
  real :: tmpobs,sd
  real(kind=8) :: jd
  integer, dimension(100) :: indexbuff
  real, dimension(100) :: buffer
  real, dimension(100,nemsteps*nbox) :: buffer_knst
  real, dimension(100,numxnest*numynest) :: buffer_kfin
  real, dimension(100,ncrs) :: buffer_kout
  real, dimension(ncrs) :: tmpbkg
  real, dimension(nemsteps*nbox) :: tmpknst
  real, dimension(numxnest*numynest) :: tmpkfin
  real, dimension(ncrs) :: tmpkout
  real, dimension(maxrows,nemsteps*nbox) :: knest
  real, dimension(maxrows,ncrs) :: kout
  real, dimension(maxrows,ncrs) :: bkgnd
  real, dimension(maxobs,nemsteps*nbox) :: knest_obs
  real, dimension(maxobs,ncrs) :: kout_obs,bkgnd_obs
  real, dimension(maxobs,6) :: allobs
  real, dimension(100000,7) :: obsdata

  open(100,file=trim(dirinvert)//'obsread.txt',status='old',action='read',iostat=ierr)
  if(ierr.gt.0) then
    write(logid,*) 'Error opening file: '//trim(dirinvert)//'obsread.txt'
    stop
  endif
  
  !---------------------------------------------------------------
  ! read observations in obsread.txt 
  !---------------------------------------------------------------

  i=0
  do while(ierr.eq.0) 
    i=i+1
    read(100,fmt='(A7,1X,I5,1X,I8,1X,I5,1X,F6.2,1X,F7.2,1X,F8.2,1X,F10.4)',iostat=ierr,end=10) &
         & recid,rnum,yyyymmdd,timestep,rlat,rlon,ralt,con
    do j=1,numrec
      if(recid.eq.recname(j)) then
        nr=j
        go to 20
      endif
    end do
20  continue
    obsdata(i,1)=float(nr)
    obsdata(i,2)=float(rnum)
    obsdata(i,3)=float(timestep)
    obsdata(i,4)=rlat
    obsdata(i,5)=rlon
    obsdata(i,6)=ralt
    obsdata(i,7)=con
  end do

10 continue

  close(100)

  !---------------------------------------------------------------
  ! calculate averages
  !---------------------------------------------------------------

  ! for stationary receptors average observations and corresponding rows of kernel 
  ! matrix to obstep but no averaging for moving receptors (ships and aircraft)

  ind=0
  do nr=1,numrec
    if(trim(rectyp(nr)).eq.'FM'.or.trim(rectyp(nr)).eq.'CM') then
      ! stationary receptor
      jd=juldatei
      do while(jd.lt.juldatef)
        tmpobs=0.
        tmpbkg(:)=0.
        tmpknst(:)=0.
        tmpkfin(:)=0.
        tmpkout(:)=0.
        timestep=int((jd-juldatei)*24.)
        ! loop over observations
        cntobs=0
        buffer(:)=0.
        indexbuff(:)=0
        do i=1,size(obsdata,1)
          if(int(obsdata(i,1)).eq.nr) then
            ! observation belongs to current receptor
            if(int(obsdata(i,3)).ge.timestep.and.int(obsdata(i,3)).lt.(timestep+int(obstep*24.))) then
              ! observation belongs to current timestep
              if(obsfilter.and.trim(rectyp(nr)).eq.'CM') then
                ! filter observations
                ! local time
                hh=int(obsdata(i,3)-floor(obsdata(i,3)/24.)*24.)
                hloc=hh+int(obsdata(i,5)*24./360.)
                if(hloc.lt.0) hloc=hloc+24
                if(hloc.gt.24) hloc=hloc-24
                if(obsdata(i,6).gt.1000) then
                  ! select nighttime for mountain sites
                  if(hloc.lt.6) then
                    cntobs=cntobs+1
                    buffer(cntobs)=obsdata(i,7)
                    indexbuff(cntobs)=obsdata(i,3) 
                  endif
                else
                  ! select afternoon for low altitude sites
                  if(hloc.ge.12.and.hloc.lt.16) then
                    cntobs=cntobs+1
                    buffer(cntobs)=obsdata(i,7)
                    indexbuff(cntobs)=obsdata(i,3)
                  endif
                endif
              else
                ! do not filter observations
                cntobs=cntobs+1
                buffer(cntobs)=obsdata(i,7)
                indexbuff(cntobs)=obsdata(i,3)
              endif ! obsfilter
            endif ! timestep          
          endif ! nr
        end do ! i
 
        if(cntobs.ne.0) then
          ! average observations for current timestep
          tmpobs=sum(buffer(1:cntobs))/float(cntobs) 
          call stdev(cntobs,buffer(1:cntobs),sd)
        else
          ! no observations for current timestep/receptor
          go to 50
        endif

        if(calcbg) then
          ! loop over background values
          cnt=0
          do i=1,maxrows
            if(indexbkg(i,1).eq.nr) then
              ! background belongs to current receptor
              if(any(indexbuff(1:cntobs).eq.indexbkg(i,2))) then
                ! row belongs to an observation in this timestep
                cnt=cnt+1
                tmpbkg=tmpbkg+bkgnd(i,:)
              endif
            endif
          end do ! i
          if(cnt.ne.0) then
            ! average background for current timestep/receptor
            tmpbkg=tmpbkg/float(cnt)
          else
            ! no background for current timestep
            go to 50
          endif
        endif

        ! loop over rows in kernel matrix
        cnt=0
        do i=1,maxrows
          if(indexobs(i,1).eq.nr) then
            ! row belongs to current receptor
            if(any(indexbuff(1:cntobs).eq.indexobs(i,2))) then
              ! row belongs to an observation in this timestep
              cnt=cnt+1
              buffer_knst(cnt,:)=knest(i,:)
              if(.not.lowmemory) then
                buffer_kfin(cnt,:)=knest_fin(i,:)
              endif
              buffer_kout(cnt,:)=kout(i,:)
            endif
          endif 
        end do ! i
        if(cnt.ne.0) then
          ! average rows for current timestep
          tmpknst=sum(buffer_knst(1:cnt,:),dim=1)/float(cnt)
          if(.not.lowmemory) then
            tmpkfin=sum(buffer_kfin(1:cnt,:),dim=1)/float(cnt)
          endif
          tmpkout=sum(buffer_kout(1:cnt,:),dim=1)/float(cnt)
        else
          ! no rows for current timestep
          go to 50
        endif

        ! assign observations to allobs
        ind=ind+1
        allobs(ind,1)=float(nr)
        allobs(ind,2)=float(timestep)
        allobs(ind,3)=tmpobs
        allobs(ind,4)=0.
        allobs(ind,5)=sqrt(measerr**2+bkgerr**2+sd**2)
        if(calcbg) allobs(ind,6)=sum(tmpbkg)
        ! assign rows to bkgnd_obs, knest_obs, knest_finobs, kout_obs
        bkgnd_obs(ind,:)=tmpbkg
        knest_obs(ind,:)=tmpknst
        if(.not.lowmemory) then
          knest_finobs(ind,:)=tmpkfin
        endif
        kout_obs(ind,:)=tmpkout
50      continue
        jd=jd+obstep
      end do ! jd

    else

      ! moving receptor (ships or aircraft)
      ! loop over observations
      ! note: assumes flexpart has been run for these observations
      do i=1,size(obsdata,1)
        if(int(obsdata(i,1)).eq.nr) then
          ! observation belongs to current receptor      
          ind=ind+1
          allobs(ind,1)=float(nr)
          allobs(ind,2)=obsdata(i,3)
          allobs(ind,3)=obsdata(i,7)
          allobs(ind,4)=0.
          allobs(ind,5)=measerr
          if(calcbg) then
            ! find matching row of bkgnd matrix
            do j=1,maxrows
              if(indexbkg(j,1).eq.nr.and.indexbkg(j,2).eq.int(obsdata(i,2))) then
                allobs(ind,6)=sum(bkgnd(j,:))
                bkgnd_obs(ind,:)=bkgnd(j,:)
              endif
            end do ! j
          endif
          ! find matching row of kernel matrix
          do j=1,maxrows
            if(indexobs(j,1).eq.nr.and.indexobs(j,2).eq.int(obsdata(i,2))) then
              knest_obs(ind,:)=knest(j,:)  
              if(.not.lowmemory) then
                knest_finobs(ind,:)=knest_fin(j,:)
              endif
              kout_obs(ind,:)=kout(j,:)
            endif
          end do ! j
        endif ! nr
      end do ! i 
    endif ! rectyp
  end do ! nr


end subroutine averageobs

