!---------------------------------------------------------------------------------------
! FLEXINVERT : calcprior
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Calculates prior modelled mixing ratios and the background mixing ratios
!!
!---------------------------------------------------------------------------------------

subroutine calcprior(knest_obs,knest_trim,kout_obs,&
                     &bkgnd_obs,xb,allobs,yobs,ymod,ybgi,yunc,ydat,yrec,nobs)

  use mod_var
  use mod_dates

  implicit none

  integer :: i,j,k,nobs,ierr
  real, dimension(nbox*nemsteps+nbkgnd) :: xb
  real, dimension(maxobs,6) :: allobs
  real, dimension(maxobs) :: yobs,ymod,ybgi,yunc
  integer, dimension(maxobs) :: ydat
  character(len=7), dimension(maxobs) :: yrec
  real, dimension(maxobs,nemsteps*nbox+nbkgnd) :: knest_trim
  real, dimension(maxobs,nemsteps*nbox) :: knest_obs
  real, dimension(maxobs,ncrs) :: bkgnd_obs
  real, dimension(maxobs,ncrs) :: kout_obs

  write(logid,*) 'Calculating prior model mixing ratios...'

  ymod(:)=0.
  yobs(:)=0.
  ybgi(:)=0.
  knest_trim(:,:)=0.
  if(.not.lowmemory) then
    knest_fintrim(:,:)=0.
  endif

  nobs=0
  do i=1,maxobs
    ! check for missing value
    if(allobs(i,3).eq.-999) go to 50
    if(calcbg) then
      if(.not.optimizbg) then
        ! subtract background from observations
        ! if observation lower than background cut observation (passive tracer no sink)
        if((allobs(i,3)-allobs(i,6)).lt.0.) go to 50
        nobs=nobs+1
        yobs(nobs)=allobs(i,3)-allobs(i,6)-sum(kout_obs(i,:))
        ybgi(nobs)=allobs(i,6)+sum(kout_obs(i,:))
        knest_trim(nobs,:)=knest_obs(i,:)
        ymod(nobs)=dot_product(knest_trim(nobs,:),xb(:))
      else
        ! do not subtract background from observations
        ! calculate part of kernel matrix for background optimization
        nobs=nobs+1
        yobs(nobs)=allobs(i,3)
        knest_trim(nobs,1:nbox*nemsteps)=knest_obs(i,:)
        knest_trim(nobs,nbox*nemsteps+1:nbox*nemsteps+nbkgnd)=kout_obs(i,:)+bkgnd_obs(i,:)
        ymod(nobs)=dot_product(knest_trim(nobs,:),xb(:))
        ybgi(nobs)=dot_product(knest_trim(nobs,nbox*nemsteps+1:nbox*nemsteps+nbkgnd),xb(nbox*nemsteps+1:nbox*nemsteps+nbkgnd))
      endif ! optimizbg
    else
      ! assume observations are departures from background
      nobs=nobs+1
      yobs(nobs)=allobs(i,3)
      ybgi(nobs)=0.
      knest_trim(nobs,1:nbox*nemsteps)=knest_obs(i,:)
      ymod(nobs)=dot_product(knest_trim(nobs,:),xb(:))
    endif ! calcbg
    if(.not.lowmemory) then
      knest_fintrim(nobs,:)=knest_finobs(i,:)
    endif
    ydat(nobs)=caldate(juldatei+(allobs(i,2)/24.))
    yunc(nobs)=allobs(i,5)
    yrec(nobs)=recname(int(allobs(i,1)))
50  continue
  end do ! i

  write(logid,fmt='(1X,A29,I5)'), 'Total number of observations:',nobs

end subroutine calcprior

