!---------------------------------------------------------------------------------------
! FLEXINVERT : initconc
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Calls a subroutine to read global fields of initial concentration from files and
!! converts the mass mixing ratios from these to the FLEXPART grid
!---------------------------------------------------------------------------------------

subroutine initconc(outheight,ntmax,numpoint,jd)

  use mod_var
  use mod_dates

  implicit none

  include 'netcdf.inc'

  character(len=100) :: fileout,dname1,dname2,dname3,varunit,varname,comment
  character(len=4) :: yyyystr,mmstr
  integer :: yyyymmddhh,yyyymmdd,yyyy,mm,it,iz,jy,ix
  real(kind=8) :: jd
  integer :: numpoint,itime,ntmax
  real, dimension(numxgrid,numygrid,numzgrid,numpoint) :: grid
  real, dimension(nzmax) :: outheight
  real, dimension(numxgrid) :: outlon
  real, dimension(numygrid) :: outlat
  real, dimension(numzgrid) :: presout
  real, dimension(:), allocatable :: height
  real, dimension(:,:,:), allocatable :: immrin,immrout,immrinz,immroutz,immrsave
  real :: tz,pz

  !--------------------------------------------------------------
  ! read initial concentration fields
  !--------------------------------------------------------------

  ! select model output to read
!  call readlmdz(jd)
!  call readpyvar(jd)
!  call readtm5(jd)
  call readmacc(jd)
!  call readnoaa(jd)

  !--------------------------------------------------------------
  ! convert initial mixing ratios to flexpart grid
  !--------------------------------------------------------------

  allocate(immrin(nxini,nyini,nzini))
  allocate(immrout(nxini,nyini,numzgrid))
  allocate(immrinz(nxini,nyini,1))
  allocate(immroutz(numxgrid,numygrid,1))
  allocate(immr(numxgrid,numygrid,numzgrid,nday))
  allocate(immrsave(numxgrid,numygrid,numzgrid))

  immrout(:,:,:)=0.
  immroutz(:,:,1)=0.

  print*, 'lonini:',lonini
  print*, 'latini:',latini

  ! grid conversions
  do it=1,nday
    ! interpolate vertically
    immrin=immrraw(:,:,:,it)
    call interpz(outheight,immrin,immrout)
    ! interpolate horizontally
    do iz=1,numzgrid
      immrinz(:,:,1)=immrout(:,:,iz)
      call convertgrid(.false.,1,nxini,nyini,lonini,latini,1,immrinz,immroutz)
      immr(:,:,iz,it)=immroutz(:,:,1) 
    end do 
  end do 

  ! calculate pressure out 
  tz=0.
  do iz=1,numzgrid
    tz=max(tz,ts-lrate*outheight(iz))
    pz=ps*exp(-outheight(iz)*mmol*gc/(gasc*tz))
    presout(iz)=pz
  end do

  ! for testing save mixing ratios on flexpart grid
  fileout='immr.nc'
  varname='tracer'
  varunit='vmr'
  dname1='longitude'
  dname2='latitude'
  dname3='pressure'
  comment='initial mixing ratios on flexpart grid'
  do ix=1,numxgrid
    outlon(ix)=outlon0+(ix-1)*dxout
  end do
  do jy=1,numygrid
    outlat(jy)=outlat0+(jy-1)*dyout
  end do
  immrsave=immr(:,:,:,1)
  call write3Dncdf(dirinvert,fileout,varname,varunit,immrsave,&
                   &numxgrid,numygrid,numzgrid,outlon,outlat,presout,&
                   &dname1,dname2,dname3,comment)

  deallocate(lonini)
  deallocate(latini)
  deallocate(geoh)
  deallocate(immrraw)
  deallocate(immrin)
  deallocate(immrout)
  deallocate(immrinz)
  deallocate(immroutz)
  deallocate(immrsave)

end subroutine initconc

