!---------------------------------------------------------------------------------------
! FLEXINVERT : reademissions
!---------------------------------------------------------------------------------------
!> Reads prior emissions from a netcdf file
!---------------------------------------------------------------------------------------

subroutine reademissions()

  use mod_var

  implicit none

  include 'netcdf.inc'

  integer :: nid,ierr,xid,yid,zid,lonid,latid,tid,varid,iy
  integer, dimension(:), allocatable :: indy
  real, dimension(:), allocatable :: lato

  ! open netcdf file
  write(logid,*) 'Reading file: '//trim(fileemis)
  ierr=nf_open(trim(fileemis),NF_NOWRITE,nid)
  if(ierr.ne.0) then
    write(logid,*) 'Error opening file: ',trim(fileemis)
    stop
  endif

  ! inquire about variables
  ierr=nf_inq_dimid(nid,'longitude',xid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension: longitude'
    stop
  endif
  ierr=nf_inq_dimid(nid,'latitude',yid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension: latitude'
    stop
  endif
  ierr=nf_inq_dimid(nid,'time',zid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension: time'
    stop
  endif
  ! read dimension sizes
  ierr=nf_inq_dimlen(nid,xid,nxemi)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension length'
    stop
  endif
  ierr=nf_inq_dimlen(nid,yid,nyemi)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension length'
    stop
  endif
  ierr=nf_inq_dimlen(nid,zid,ntemi)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension length'
    stop
  endif

  allocate(lato(nyemi))
  allocate(indy(nyemi))
  allocate(lonemi(nxemi))
  allocate(latemi(nyemi))
  allocate(timemi(ntemi))
  allocate(emis(nxemi,nyemi,ntemi))

  ! read variables
  ierr=nf_inq_varid(nid,'longitude',lonid)
  ierr=nf_get_var_real(nid,lonid,lonemi)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading variable: longitude'
    stop
  endif
  ierr=nf_inq_varid(nid,'latitude',latid)
  ierr=nf_get_var_real(nid,latid,latemi)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading variable: latitude'
    stop
  endif
  ierr=nf_inq_varid(nid,'time',tid)
  ierr=nf_get_var_real(nid,tid,timemi)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading variable: time'
    stop
  endif
  ierr=nf_inq_varid(nid,trim(emisname),varid)
  ierr=nf_get_var_real(nid,varid,emis)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading variable: ',trim(emisname)
    stop
  endif
  ! if latitude order N-S convert to S-N
  lato=latemi
  call sort(nyemi,lato)
  ! indices to lat
  do iy=1,nyemi
    indy(iy)=minloc(lato,dim=1,mask=lato.eq.latemi(iy))
  end do
  emis=emis(:,indy,:)
  latemi=lato

  ierr=nf_close(nid)

  deallocate(lato)
  deallocate(indy)

end subroutine reademissions
