!---------------------------------------------------------------------------------------
! FLEXINVERT : readice
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Reads ice and snow mask from a netcdf file
!---------------------------------------------------------------------------------------

subroutine readice()

  use mod_var

  implicit none

  include 'netcdf.inc'

  integer :: nid,ierr,xid,yid,zid,lonid,latid,varid
  integer, dimension(:,:,:), allocatable :: tmp

  ! open netcdf file
  write(logid,*) 'Reading file: '//trim(fileice)
  ierr=nf_open(trim(fileice),NF_NOWRITE,nid)
  if(ierr.ne.0) then
    write(logid,*) 'Error opening file: '//trim(fileice)
    stop
  endif

  ! inquire about variables
  ierr=nf_inq_dimid(nid,'longitude',xid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension: longitude'
    stop
  endif
  ierr=nf_inq_dimid(nid,'latitude',yid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension: latitude'
    stop
  endif
  ierr=nf_inq_dimid(nid,'time',zid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension: time'
    stop
  endif
  ! read dimension sizes
  ierr=nf_inq_dimlen(nid,xid,nxice)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension length'
    stop
  endif
  ierr=nf_inq_dimlen(nid,yid,nyice)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension length'
    stop
  endif
  ierr=nf_inq_dimlen(nid,zid,ntice)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension length'
    stop
  endif

  allocate(latice(nyice))
  allocate(lonice(nxice))
  allocate(ice(nxice,nyice,1))
  allocate(tmp(nxice,nyice,ntice))

  ! read variables
  ierr=nf_inq_varid(nid,'latitude',yid)
  ierr=nf_get_var_real(nid,yid,latice)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading variable: latitude'
    stop
  endif
  ierr=nf_inq_varid(nid,'longitude',xid)
  ierr=nf_get_var_real(nid,xid,lonice)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading variable: longitude'
    stop
  endif
  ierr=nf_inq_varid(nid,trim(icename),varid)
  ierr=nf_get_var_int(nid,varid,tmp)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading variable: ',trim(icename)
    stop
  endif

  ! only permanently frozen grid cells 
  ice(:,:,1)=minval(tmp,dim=3)

  ierr=nf_close(nid)

  deallocate(tmp)

end subroutine readice

