!---------------------------------------------------------------------------------------
! FLEXINVERT : readmacc 
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Reads mixing ratios from global fields from TM5 model runs for MACC 
!! from P. Bergamaschi
!---------------------------------------------------------------------------------------

subroutine readmacc(jd)

  use mod_var
  use mod_dates

  implicit none

  include 'netcdf.inc'

  character(len=100) :: filename,filepart1,filepart2,filepart3
  character(len=2) :: mmstr
  character(len=4) :: yyyystr
  integer :: ind1,ind2
  integer :: i,ix,jy,kz,cnt,ierr
  integer :: yyyymmddhh,yyyymmdd,yyyymm,yyyy,mm,dd
  integer :: nid,xid,yid,zid,tid,varid
  real :: eomday
  real :: prs1,prs2,tmp
  real(kind=8) jd,jdi,jdf,jdt
  real, dimension(:), allocatable :: ap,bp
  real, dimension(:,:,:), allocatable :: psurf
  real, dimension(:,:,:,:), allocatable :: temp,immrtmp

  ! TM5 latitude and longitude (southern and western edges)
  ! glb600x400
  allocate(lonini(60))
  lonini=(/&
   -180,-174,-168,-162,-156,-150,-144,-138,-132,-126,&
    -120,-114,-108,-102,-96,-90,-84,-78,-72,-66,& 
    -60,-54,-48,-42,-36,-30,-24,-18,-12,-6,&
    0,6,12,18,24,30,36,42,48,54,60,&
    66,72,78,84,90,96,102,108,114,120,&  
    126,132,138,144,150,156,162,168,174/)
  allocate(latini(45))
  latini=(/&
    -90,-86,-82,-78,-74,-70,-66,-62,-58,-54,&
    -50,-46,-42,-38,-34,-30,-26,-22,-18,-14,&
    -10,-6,-2,2,6,10,14,18,22,26,&
    30,34,38,42,46,50,54,58,62,66,&
    70,74,78,82,86/)

  ! calculate start date
  if((jd-trajlen).ge.juldatei) then 
    jdi=jd-trajlen
  else
    jdi=jd
  endif
  yyyymmddhh=caldate(jdi)
  yyyymm=yyyymmddhh/10000
  jdi=juldate(yyyymm*100+1,0)

  ! calculate end date 
  yyyymmddhh=caldate(jd)
  yyyymm=yyyymmddhh/10000
  eomday=calceomday(yyyymm)
  jdf=jd+eomday-1
  yyyymmddhh=caldate(jdf)

  nday=int(jdf-jdi)+1

  ! initialize 
  jdt=jdi
  i=0
  cnt=0

  ! loop over monthly files

  do while(jdt.le.jdf)

    i=i+1

    ! define exact filename
    yyyymmddhh=caldate(jdt)
    yyyymmdd=yyyymmddhh/100
    yyyymm=yyyymmdd/100
    yyyy=yyyymm/100
    mm=(yyyymmddhh-yyyy*1000000)/10000
    ind1=index(fileimmr,'YYYY')-1
    filepart1=fileimmr(1:ind1)
    ind2=index(fileimmr,'MM')-1
    filepart2=fileimmr((ind1+5):ind2)
    filepart3=fileimmr((ind2+3):len(fileimmr))
    write(yyyystr,'(i4)') yyyy
    write(mmstr,'(i2.2)') mm
    filename=trim(filepart1)//yyyystr//trim(filepart2)//mmstr//trim(filepart3)

    ! open netcdf file
    write(logid,*) 'Reading the file: '//trim(filename)
    ierr=nf_open(trim(filename),NF_NOWRITE,nid)
    if(ierr.ne.0) then
      write(logid,*) 'Error opening file: '//trim(filename)
      stop
    endif

    ! inquire about dimensions
    ierr=nf_inq_dimid(nid,'longitude',xid)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension: longitude'
      stop
    endif
    ierr=nf_inq_dimid(nid,'latitude',yid)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension: latitude'
      stop
    endif
    ierr=nf_inq_dimid(nid,'level',zid)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension: level'
      stop
    endif
    ierr=nf_inq_dimid(nid,'time',tid)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension: time'
      stop
    endif

    ! read dimension sizes
    ierr=nf_inq_dimlen(nid,xid,nxini)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension length'
      stop
    endif
    ierr=nf_inq_dimlen(nid,yid,nyini)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension length'
      stop
    endif
    ierr=nf_inq_dimlen(nid,zid,nzini)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension length'
      stop
    endif
    ierr=nf_inq_dimlen(nid,tid,ntini)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension length'
      stop
    endif

    ! allocate variables 
    if(i.eq.1) then
      ! allocate only first time
      allocate(ap(nzini))
      allocate(bp(nzini))
      allocate(psurf(nxini,nyini,ntini))
      allocate(temp(nxini,nyini,nzini,ntini))
      allocate(immrraw(nxini,nyini,nzini,nday))
    endif
    ! allocate each time
    allocate(immrtmp(nxini,nyini,nzini,ntini))

    ! read variables
    if(i.eq.1) then
      ! read only first time
      ierr=nf_inq_varid(nid,'hya',varid)
      ierr=nf_get_var_real(nid,varid,ap)
      if(ierr.ne.0) then
        write(logid,*) 'Error reading variable: ap'
        stop
      endif
      ierr=nf_inq_varid(nid,'hyb',varid)
      ierr=nf_get_var_real(nid,varid,bp)
      if(ierr.ne.0) then
        write(logid,*) 'Error reading variable: bp'
        stop
      endif
      ierr=nf_inq_varid(nid,'ps',varid)
      ierr=nf_get_var_real(nid,varid,psurf)
      if(ierr.ne.0) then
        write(logid,*) 'Error reading variable: ps'
        stop
      endif
      ierr=nf_inq_varid(nid,'tempm',varid)
      ierr=nf_get_var_real(nid,varid,temp)
      if(ierr.ne.0) then
        write(logid,*) 'Error reading variable: tempm'
        stop
      endif
    endif
    ! read each time
    ierr=nf_inq_varid(nid,trim(bginame),varid)
    ierr=nf_get_var_real(nid,varid,immrtmp)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading variable: ',trim(bginame)
      stop
    endif

    ierr=nf_close(nid)

    ! convert from ppb to volume mixing ratio
    immrtmp=immrtmp/1.e9
    immrraw(:,:,:,(cnt+1):(cnt+ntini))=immrtmp
    deallocate(immrtmp)

    eomday=calceomday(yyyymm)
    jdt=jdt+eomday
    cnt=cnt+ntini
 
  enddo

  ! calculate geopotential height for each gridcell
  allocate(geoh(nxini,nyini,nzini))
  do kz=1,nzini
    do jy=1,nyini 
      do ix=1,nxini
        prs1=sum(psurf(ix,jy,:))/real(size(psurf,3))
        prs2=ap(kz)+bp(kz)*sum(psurf(ix,jy,:))/real(size(psurf,3))
        tmp=sum(temp(ix,jy,kz,:))/real(size(temp,4))
        geoh(ix,jy,kz)=(gasc*tmp/gc/mmol)*log(prs1/prs2)
      enddo
    enddo
  enddo 

  ! calculate timestep as decimal year
  allocate(timini(nday))
  do i=1,nday
    yyyymmddhh=caldate(jdi+i-1)
    yyyy=yyyymmddhh/1000000
    mm=(yyyymmddhh-1000000*yyyy)/10000
    dd=(yyyymmddhh-1000000*yyyy-10000*mm)/100
    timini(i)=float(yyyy)+float(mm-1)/12.+float(dd-1)/365.
  end do

  ! prepare to exit
  deallocate(ap)
  deallocate(bp)
  deallocate(psurf)
  deallocate(temp)

end subroutine readmacc

