!---------------------------------------------------------------------------------------
! FLEXINVERT : readnboxxy
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Reads nbox_xy definition from a netcdf file
!---------------------------------------------------------------------------------------

subroutine readnboxxy()

  use mod_var

  implicit none

  include 'netcdf.inc'
 
  integer :: nid,ierr,xid,yid,varid
  integer :: nxbox,nybox

  ! open netcdf file
  write(logid,*) 'Reading nbox_xy.nc'
  ierr=nf_open(trim(dirinvert)//'nbox_xy.nc',NF_NOWRITE,nid)
  if(ierr.ne.0) then
    write(logid,*) 'Error opening file: nbox_xy.nc'
    stop
  endif

  ! inquire about variables
  ierr=nf_inq_dimid(nid,'dimx',xid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension: dimx'
    stop
  endif
  ierr=nf_inq_dimid(nid,'dimy',yid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension: dimy'
    stop
  endif
  ! read dimension sizes
  ierr=nf_inq_dimlen(nid,xid,nxbox)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension length'
    stop
  endif
  ierr=nf_inq_dimlen(nid,yid,nybox)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension length'
    stop
  endif

  ! check dimensions are correct
  if(nxbox.ne.numxnest) then
    write(logid,*) 'Error dimension of nbox_xy'
    stop
  endif
  if(nybox.ne.numynest) then
    write(logid,*) 'Error dimension of nbox_xy'
    stop
  endif

  ! read variables
  ierr=nf_inq_varid(nid,'nbox_xy',varid)
  ierr=nf_get_var_int(nid,varid,nbox_xy)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading variable: nbox_xy'
    stop
  endif

  ierr=nf_close(nid)

  nbox=maxval(nbox_xy)

end subroutine readnboxxy

