!---------------------------------------------------------------------------------------
! FLEXINVERT : readnoaa 
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Reads mixing ratios from global fields from TM5 model runs for MACC 
!! from P. Bergamaschi
!---------------------------------------------------------------------------------------

subroutine readnoaa(jd)

  use mod_var
  use mod_dates

  implicit none

  include 'netcdf.inc'

  character(len=100) :: filename,filepart1,filepart2,filepart3
  character(len=2) :: mmstr
  character(len=4) :: yyyystr
  integer :: ind1,ind2
  integer :: n,i,ix,jy,kz,cnt,ierr
  integer :: yyyymmddhh,yyyymmdd,yyyymm,yyyy,mm,dd
  integer :: nid,xid,yid,zid,tid,varid
  real :: eomday
  real(kind=8) jd,jdi,jdf,jdt
  real, dimension(:), allocatable :: pres
  real, dimension(:,:,:,:), allocatable :: immrtmp

  ! calculate start date
  if((jd-trajlen).ge.juldatei) then 
    jdi=jd-trajlen
  else
    jdi=jd
  endif
  yyyymmddhh=caldate(jdi)
  yyyymm=yyyymmddhh/10000
  jdi=juldate(yyyymm*100+1,0)

  ! calculate end date 
  yyyymmddhh=caldate(jd)
  yyyymm=yyyymmddhh/10000
  eomday=calceomday(yyyymm)
  jdf=jd+eomday-1
  yyyymmddhh=caldate(jdf)

  nday=int(jdf-jdi)+1


  ! initialize 
  jdt=jdi
  i=0
  cnt=0

  ! loop over monthly files

  do while(jdt.le.jdf)

    i=i+1

    ! define exact filename
    yyyymmddhh=caldate(jdt)
    yyyymmdd=yyyymmddhh/100
    yyyymm=yyyymmdd/100
    yyyy=yyyymm/100
    mm=(yyyymmddhh-yyyy*1000000)/10000
    ind1=index(fileimmr,'YYYY')-1
    filepart1=fileimmr(1:ind1)
    ind2=index(fileimmr,'MM')-1
    filepart2=fileimmr((ind1+5):ind2)
    filepart3=fileimmr((ind2+3):len(fileimmr))
    write(yyyystr,'(i4)') yyyy
    write(mmstr,'(i2.2)') mm
    filename=trim(filepart1)//yyyystr//trim(filepart2)//mmstr//trim(filepart3)

    ! open netcdf file
    write(logid,*) 'Reading the file: '//trim(filename)
    ierr=nf_open(trim(filename),NF_NOWRITE,nid)
    if(ierr.ne.0) then
      write(logid,*) 'Error opening file: '//trim(filename)
      stop
    endif

    ! inquire about dimensions
    ierr=nf_inq_dimid(nid,'longitude',xid)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension: longitude'
      stop
    endif
    ierr=nf_inq_dimid(nid,'latitude',yid)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension: latitude'
      stop
    endif
    ierr=nf_inq_dimid(nid,'pressure',zid)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension: pressure'
      stop
    endif

    ! read dimension sizes
    ierr=nf_inq_dimlen(nid,xid,nxini)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension length'
      stop
    endif
    ierr=nf_inq_dimlen(nid,yid,nyini)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension length'
      stop
    endif
    ierr=nf_inq_dimlen(nid,zid,nzini)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension length'
      stop
    endif

    ! allocate variables 
    if(i.eq.1) then
      ! allocate only first time
      allocate(immrraw(nxini,nyini,nzini,nday))
      allocate(lonini(nxini))
      allocate(latini(nyini))
      allocate(pres(nzini))
    endif
    ! allocate each time
    allocate(immrtmp(nxini,nyini,nzini,1))

    ! read variables
    if(i.eq.1) then
      ! read only first time
      ierr=nf_inq_varid(nid,'longitude',varid)
      ierr=nf_get_var_real(nid,varid,lonini)
      if(ierr.ne.0) then
        write(logid,*) 'Error reading variable: longitude'
        stop
      endif
      ierr=nf_inq_varid(nid,'latitude',varid)
      ierr=nf_get_var_real(nid,varid,latini)
      if(ierr.ne.0) then
        write(logid,*) 'Error reading variable: latitude'
        stop
      endif
      ierr=nf_inq_varid(nid,'pressure',varid)
      ierr=nf_get_var_real(nid,varid,pres)
      if(ierr.ne.0) then
        write(logid,*) 'Error reading variable: pressure'
        stop
      endif
    endif
    ! read each time
    ierr=nf_inq_varid(nid,trim(bginame),varid)
    ierr=nf_get_var_real(nid,varid,immrtmp)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading variable: ',trim(bginame)
      stop
    endif

    ierr=nf_close(nid)

    ! convert from ppb to volume mixing ratio
    immrtmp=immrtmp/1.e9

    eomday=calceomday(yyyymm)
    do n=1,int(eomday)
      immrraw(:,:,:,(cnt+n))=immrtmp(:,:,:,1)
    end do
    deallocate(immrtmp)

    jdt=jdt+eomday
    cnt=cnt+eomday

  enddo

  ! calculate geopotential height for each gridcell
  allocate(geoh(nxini,nyini,nzini))
  do kz=1,nzini
    do jy=1,nyini 
      do ix=1,nxini
        geoh(ix,jy,kz)=(gasc*ts/gc/mmol)*log(ps/pres(kz))
      enddo
    enddo
  enddo 

  ! calculate timestep as decimal year
  allocate(timini(nday))
  do i=1,nday
    yyyymmddhh=caldate(jdi+i-1)
    yyyy=yyyymmddhh/1000000
    mm=(yyyymmddhh-1000000*yyyy)/10000
    dd=(yyyymmddhh-1000000*yyyy-10000*mm)/100
    timini(i)=float(yyyy)+float(mm-1)/12.+float(dd-1)/365.
  end do

  ! prepare to exit
  deallocate(pres)

end subroutine readnoaa

