!---------------------------------------------------------------------------------------
! FLEXINVERT : readobs
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Reads observations for each receptor and writes these to the observation file
!! for each release timestep of the backwards runs
!---------------------------------------------------------------------------------------

subroutine readobs()

  use mod_var
  use mod_dates

  implicit none
  
  character(len=100) :: filename,dump
  character(len=4) :: yrstr
  integer :: obsid,n,nr,rnum,ierr
  integer :: yyyymmddhh,yyyymmdd,yyyy,hhmiss,timestep
  integer :: yr,mn,dd,hh,mi,ss
  real :: rlon,rlat,ralt,con,sd,err
  real(kind=8) :: julday
 
  write(logid,*) 'Reading observations...'

  ! file to write observations to

  obsid=300
  open(obsid,file=trim(dirinvert)//'obsread.txt',status='replace',action='write',iostat=ierr)

  ! loop over stations/receptors

  do nr=1,numrec

    ! modify the lines below for different directory structures and/or file names
    if(trim(recfreq(nr)).eq.'ANNUAL') then
      yyyymmddhh=caldate(juldatei)
      yyyy=yyyymmddhh/1e6
      write(yrstr,'(i4)') yyyy
      filename=yrstr//'/'//trim(recname(nr))//'_'//trim(specname)//'_'//trim(rectyp(nr))//'_'//yrstr//'.dat'
    else
      filename=trim(recname(nr))//'_'//trim(specname)//'_'//trim(rectyp(nr))//'.dat'
    endif
 
    open(100,file=trim(dirobs)//trim(filename),action='read',status='old',iostat=ierr)
    if(ierr.gt.0) write(logid,*) 'Error opening file: '//trim(dirobs)//trim(filename)
    if(ierr.eq.0) write(logid,*) 'Reading file: '//trim(dirobs)//trim(filename)

    ! header has 21 lines
    do n=1,21
      read(100,fmt='(A100)',iostat=ierr) dump
    end do

    do while(ierr.eq.0)

      ! modify the lines below for different observation file formats
      read(100,fmt='(I4,1X,I2,1X,I2,1X,I2,1X,I2,1X,I2,F6.2,1X,F7.2,1X,F8.2,1X,F10.4)',iostat=ierr,end=10) &
           &yr,mn,dd,hh,mi,ss,rlat,rlon,ralt,con

      if(con.eq.missval) go to 10

      ! calculate julday 
      yyyymmdd=yr*10000+mn*100+dd
      hhmiss=((hh/relperiod)+1)*relperiod*10000
      julday=juldate(yyyymmdd,hhmiss)
      timestep=int((julday-juldatei)*24.)

      ! if in relevant timestep write to obsread.txt 
      if(julday.ge.juldatei.and.julday.lt.juldatef) then
        rnum=rnum+1
        write(obsid,fmt='(A7,1X,I5,1X,I8,1X,I5,1X,F6.2,1X,F7.2,1X,F8.2,1X,F10.4)') &
              &recname(nr),rnum,yyyymmdd,timestep,rlat,rlon,ralt,con
      endif ! julday

10    continue

    end do ! ierr

    close(100)

  enddo ! numrec

  close(obsid)

end subroutine readobs   

