!---------------------------------------------------------------------------------------
! FLEXINVERT : readpriorerror
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Reads prior flux error from netcdf file 
!---------------------------------------------------------------------------------------

subroutine readpriorerror(outheight)

  use mod_var

  implicit none

  include 'netcdf.inc'

  integer :: ierr,nid,xid,yid,nx,ny,lonid,latid,varid,iy
  integer, dimension(numygrid) :: indy
  real, dimension(numygrid) :: lat,lato
  real, dimension(numxgrid) :: lon
  real, dimension(nzmax) :: outheight


  write(logid,*) 'Calculating prior flux errors...'

  allocate(errprior(numxgrid,numygrid))

  ! open netcdf file
  write(logid,*) 'Reading file: '//trim(fileerror)
  ierr=nf_open(trim(fileerror),NF_NOWRITE,nid)
  if(ierr.gt.0) then
    write(logid,*) 'Error opening file: ',trim(fileerror)
    stop
  endif

  ! inquire about variables
  ierr=nf_inq_dimid(nid,'longitude',xid)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading dimension: longitude'
    stop
  endif
  ierr=nf_inq_dimid(nid,'latitude',yid)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading dimension: latitude'
    stop
  endif

  ! read dimension sizes
  ierr=nf_inq_dimlen(nid,xid,nx)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading dimension length'
    stop
  endif
  ierr=nf_inq_dimlen(nid,yid,ny)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading dimension length'
    stop
  endif

  ! check dimensions
  if(nx.ne.numxgrid) then
    write(logid,*) 'Error longitude dimension wrong'
    stop
  endif
  if(ny.ne.numygrid) then
    write(logid,*) 'Error latitude dimension wrong'
    stop
  endif

  ! read variables
  ierr=nf_inq_varid(nid,'longitude',lonid)
  ierr=nf_get_var_real(nid,lonid,lon)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading variable: longitude'
    stop
  endif
  ierr=nf_inq_varid(nid,'latitude',latid)
  ierr=nf_get_var_real(nid,latid,lat)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading variable: latitude'
    stop
  endif
  ierr=nf_inq_varid(nid,trim(errorname),varid)
  ierr=nf_get_var_real(nid,varid,errprior)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading variable: ',trim(errorname)
    stop
  endif
  ! if latitude order N-S convert to S-N
  lato=lat
  call sort(ny,lato)
  ! indices to lat
  do iy=1,ny
    indy(iy)=minloc(lato,dim=1,mask=lato.eq.lat(iy))
  end do
  errprior=errprior(:,indy)

  ierr=nf_close(nid)

  ! convert to volume mixing ratio units (kg/m3/s)
  errprior=errprior*numscale/3600./outheight(1)

end subroutine readpriorerror

