!---------------------------------------------------------------------------------------
! FLEXINVERT : readtm5 
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Reads mixing ratios from global fields from TM5 model runs from P. Bergamaschi 
!---------------------------------------------------------------------------------------

subroutine readtm5(jd)

  use mod_var
  use mod_dates

  implicit none

  include 'netcdf.inc'

  character(len=100) :: filename,filepart1,filepart2,filepart3,filepart4
  character(len=2) :: ddstr,mmstr
  character(len=4) :: yyyystr
  integer :: i,kz,ix,jy
  integer :: ind1,ind2,ind3
  integer :: yyyymmddhh,yyyymmdd,yyyymm,yyyy,mm,dd
  integer :: nid,yid,xid,zid,ierr,varid
  real :: eomday
  real :: prs1,prs2,tmp
  real(kind=8) jd,jdi,jdf,jdt
  real, dimension(:,:), allocatable :: psurf
  real, dimension(:,:,:), allocatable :: temp
  real, dimension(:,:,:), allocatable :: immrtmp

  ! TM5 pressure levels
  real, dimension(26) :: ap = (/&
    0.000000,7.367743,210.393890,855.361755,&
    2063.779785,3850.913330,6144.314941,8802.356445,&
    11632.758789,14411.124023,16899.468750,18864.750000,&
    20097.402344,20429.863281,19755.109375,18045.183594,&
    15379.805664,12077.446289,8765.053711,6018.019531,&
    3960.291504,1680.640259,713.218079,298.495789,&
    95.636963,0.000000/)
  real, dimension(26) :: bp = (/&
    1.000000,0.994019,0.979663,0.951822,&
    0.907884,0.847375,0.771597,0.683269,&
    0.586168,0.484772,0.383892,0.288323,&
    0.202476,0.130023,0.073534,0.034121,&
    0.011143,0.001815,0.000076,0.000000,&
    0.000000,0.000000,0.000000,0.000000,&
    0.000000,0.000000/)

  ! TM5 latitude and longitude
  ! glb600x400
  allocate(lonini(60))
  lonini=(/&
   -180,-174,-168,-162,-156,-150,-144,-138,-132,-126,&
    -120,-114,-108,-102,-96,-90,-84,-78,-72,-66,& 
    -60,-54,-48,-42,-36,-30,-24,-18,-12,-6,&
    0,6,12,18,24,30,36,42,48,54,60,&
    66,72,78,84,90,96,102,108,114,120,&  
    126,132,138,144,150,156,162,168,174/)
  allocate(latini(45))
  latini=(/&
    -90,-86,-82,-78,-74,-70,-66,-62,-58,-54,&
    -50,-46,-42,-38,-34,-30,-26,-22,-18,-14,&
    -10,-6,-2,2,6,10,14,18,22,26,&
    30,34,38,42,46,50,54,58,62,66,&
    70,74,78,82,86/)

  ! calculate start date
  if((jd-trajlen).ge.juldatei) then 
    jdi=jd-trajlen
  else
    jdi=jd
  endif

  ! calculate end date 
  yyyymmddhh=caldate(jd)
  yyyymm=yyyymmddhh/10000
  eomday=calceomday(yyyymm)
  jdf=jd+eomday-1.

  nday=int(jdf-jdi)+1

  ! initialize 
  jdt=jdi
  i=0

  ! loop over daily files

  do while(jdt.le.jdf)

    i=i+1

    ! define exact filename
    yyyymmddhh=caldate(jdt)
    yyyymmdd=yyyymmddhh/100
    yyyy=yyyymmddhh/1000000
    mm=(yyyymmddhh-yyyy*1000000)/10000
    dd=(yyyymmddhh-yyyy*1000000-mm*10000)/100
    ind1=index(fileimmr,'YYYY')-1
    filepart1=fileimmr(1:ind1)
    ind2=index(fileimmr,'MM')-1
    filepart2=fileimmr((ind1+5):ind2)
    ind3=index(fileimmr,'DD')-1
    filepart3=fileimmr((ind2+3):ind3)
    filepart4=fileimmr((ind3+3):len(fileimmr))
    write(yyyystr,'(i4)') yyyy
    write(mmstr,'(i2.2)') mm
    write(ddstr,'(i2.2)') dd
    filename=trim(filepart1)//yyyystr//trim(filepart2)//mmstr//trim(filepart3)//ddstr//trim(filepart4)

    ! open netcdf file
    write(logid,*) 'Reading the file: '//trim(filename)
    ierr=nf_open(trim(filename),NF_NOWRITE,nid)
    if(ierr.ne.0) then
      write(logid,*) 'Error opening file: '//trim(filename)
      stop
    endif

    ! inquire about variables
    ierr=nf_inq_dimid(nid,'x',xid)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension: x'
      stop
    endif
    ierr=nf_inq_dimid(nid,'y',yid)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension: y'
      stop
    endif
    ierr=nf_inq_dimid(nid,'z',zid)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension: z'
      stop
    endif

    ! read dimension sizes
    ierr=nf_inq_dimlen(nid,xid,nxini)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension length'
      stop
    endif
    ierr=nf_inq_dimlen(nid,yid,nyini)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension length'
      stop
    endif
    ierr=nf_inq_dimlen(nid,zid,nzini)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading dimension length'
      stop
    endif

    ! allocate variables on first iteration
    if(i.eq.1) then
      allocate(psurf(nxini,nyini))
      allocate(temp(nxini,nyini,nzini))
      allocate(immrtmp(nxini,nyini,nzini))
      allocate(immrraw(nxini,nyini,nzini,nday))
    endif

    ! read variables
    if(i.eq.1) then
      ! read only first time
      ierr=nf_inq_varid(nid,'presm',varid)
      ierr=nf_get_var_real(nid,varid,psurf)
      if(ierr.ne.0) then
        write(logid,*) 'Error reading variable: presm'
        stop
      endif
      ierr=nf_inq_varid(nid,'tempm',varid)
      ierr=nf_get_var_real(nid,varid,temp)
      if(ierr.ne.0) then
        write(logid,*) 'Error reading variable: tempm'
        stop
      endif
    endif
    ! read each time
    ierr=nf_inq_varid(nid,trim(bginame),varid)
    ierr=nf_get_var_real(nid,varid,immrtmp)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading variable: ',trim(bginame)
      stop
    endif

    ierr=nf_close(nid)

    ! assign mixing ratios and divide by 1e9
    immrtmp=immrtmp/1.e9
    immrraw(:,:,:,i)=immrtmp

    jdt=jdt+1.

  end do ! jdt

  ! calculate geopotential height for each gridcell
  allocate(geoh(nxini,nyini,nzini))
  do kz=1,nzini
    do jy=1,nyini
      do ix=1,nxini
        prs1=psurf(ix,jy)
        prs2=ap(kz)+bp(kz)*psurf(ix,jy)
        tmp=temp(ix,jy,kz)
        geoh(ix,jy,kz)=(gasc*tmp/gc/mmol)*log(prs1/prs2)
      enddo
    enddo
  enddo

  ! convert time into decimal year
  allocate(timini(nday))
  do i=1,nday
    yyyymmddhh=caldate(jdi+i)
    yyyy=yyyymmddhh/1000000
    mm=(yyyymmddhh-1000000*yyyy)/10000
    dd=(yyyymmddhh-1000000*yyyy-10000*mm)/100
    timini(i)=float(yyyy)+float(mm-1)/12.+float(dd-1)/365.
  end do

  deallocate(psurf)
  deallocate(temp)
  deallocate(immrtmp)

end subroutine readtm5
