!---------------------------------------------------------------------------------------
! FLEXINVERT : readtransport
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Reads transport related files written in previous run of FLEXINVERT. 
!! The files read are:
!!  - knest.nc
!!  - kout.nc (only if calcbg is true)
!!  - knest_fin.nc (only if lowmemory is false)
!!  - indexobs.txt 
!---------------------------------------------------------------------------------------

subroutine readtransport(knest,kout)

  use mod_var

  implicit none

  include 'netcdf.inc'

  character(len=100) :: filename
  real, dimension(maxrows,nemsteps*nbox) :: knest
  real, dimension(maxrows,ncrs) :: kout
  integer :: i,nid,ierr,xid,yid,varid,ndimr,ndimbx

  !---------------------------------------------------------------
  ! read knest
  !---------------------------------------------------------------

  ! open netcdf file
  write(logid,*) 'Reading file: knest.nc'
  filename=trim(dirinvert)//'knest.nc'
  ierr=nf_open(trim(filename),NF_NOWRITE,nid)
  if(ierr.gt.0) then
    write(logid,*) 'Error opening file: ',trim(filename)
    stop
  endif
  ! inquire about variables
  ierr=nf_inq_dimid(nid,'dimr',xid)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading dimension: dimr'
    stop
  endif
  ierr=nf_inq_dimid(nid,'dimbx',yid)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading dimension: dimbx'
    stop
  endif
  ! read dimension sizes
  ierr=nf_inq_dimlen(nid,xid,ndimr)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading dimension length'
    stop
  endif
  ierr=nf_inq_dimlen(nid,yid,ndimbx)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading dimension length'
    stop
  endif
  ! check dimensions
  if(ndimr.ne.maxrows) then
    write(logid,*) 'Error: mismatching observation dimension'
    stop
  endif
  if(ndimbx.ne.nemsteps*nbox) then
    write(logid,*) 'Error: mismatching state dimension'
    stop
  endif
  ! read variable
  ierr=nf_inq_varid(nid,'knest',varid)
  ierr=nf_get_var_real(nid,varid,knest)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading variable knest'
    stop
  endif
  ierr=nf_close(nid)

  !---------------------------------------------------------------
  ! read knest_fin
  !---------------------------------------------------------------

  if(.not.lowmemory) then
    ! open netcdf file
    write(logid,*) 'Reading file: knest_fin.nc'
    filename=trim(dirinvert)//'knest_fin.nc'
    ierr=nf_open(trim(filename),NF_NOWRITE,nid)
    if(ierr.gt.0) then
      write(logid,*) 'Error opening file: ',trim(filename)
      stop
    endif
    ! inquire about variables
    ierr=nf_inq_dimid(nid,'dimr',xid)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension: dimr'
      stop
    endif
    ierr=nf_inq_dimid(nid,'dimb',yid)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension: dimb'
      stop
    endif
    ! read dimension sizes
    ierr=nf_inq_dimlen(nid,xid,ndimr)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension length'
      stop
    endif
    ierr=nf_inq_dimlen(nid,yid,ndimbx)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension length'
      stop
    endif
    ! check dimensions
    if(ndimr.ne.maxrows) then
      write(logid,*) 'Error: mismatching observation dimension'
      stop
    endif
    if(ndimbx.ne.numxnest*numynest) then
      write(logid,*) 'Error: mismatching state dimension'
      stop
    endif
    ! read variable
    ierr=nf_inq_varid(nid,'knest_fin',varid)
    ierr=nf_get_var_real(nid,varid,knest_fin)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable knest_fin'
      stop
    endif
    ierr=nf_close(nid)
  endif ! if not lowmemory
 
  !---------------------------------------------------------------
  ! read kout
  !---------------------------------------------------------------

  if(calcbg) then
    ! open netcdf file
    write(logid,*) 'Reading file: kout.nc'
    filename=trim(dirinvert)//'kout.nc'
    ierr=nf_open(trim(filename),NF_NOWRITE,nid)
    if(ierr.gt.0) then
      write(logid,*) 'Error opening file: ',trim(filename)
      stop
    endif
    ! inquire about variables
    ierr=nf_inq_dimid(nid,'dimr',xid)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension: dimr'
      stop
    endif
    ierr=nf_inq_dimid(nid,'dimcrs',yid)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension: dimcrs'
      stop
    endif
    ! read dimension sizes
    ierr=nf_inq_dimlen(nid,xid,ndimr)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension length'
      stop
    endif
    ierr=nf_inq_dimlen(nid,yid,ndimbx)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension length'
      stop
    endif
    ! check dimensions
    if(ndimr.ne.maxrows) then
      write(logid,*) 'Error: mismatching observation dimension'
      stop
    endif
    if(ndimbx.ne.ncrs) then
      write(logid,*) 'Error: mismatching state dimension'
      stop
    endif
    ! read variable
    ierr=nf_inq_varid(nid,'kout',varid)
    ierr=nf_get_var_real(nid,varid,kout)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable kout'
      stop
    endif
    ierr=nf_close(nid)
  endif

  !---------------------------------------------------------------
  ! read indexobs
  !---------------------------------------------------------------

  allocate(indexobs(maxrows,2))
  open(100,file=trim(dirinvert)//'indexobs.txt',status='old',action='read',iostat=ierr)
  if(ierr.gt.0) then
    write(logid,*) 'Error opening file: '//trim(dirinvert)//'indexobs.txt'
    stop
  endif
  i=0
  do while(ierr.eq.0)
    i=i+1
    read(100,fmt='(I2,1X,I5)',iostat=ierr,end=10) indexobs(i,1),indexobs(i,2)
  end do
10 continue

  close(100)
 
end subroutine readtransport

