!---------------------------------------------------------------------------------------
! FLEXINVERT : save
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Saves data to netcdf and ascii files
!---------------------------------------------------------------------------------------

subroutine save(nobs,knest_trim,covr,covagg,&
                &cova,gain,xb,x,yrec,ydat,yobs,ymod,ybgi,&
                &yunc,ebgb,ebga,modout,nmod,timeout)

  use mod_var
  
  implicit none

  character(len=100) :: fileout,varunit,comment
  character(len=100) :: varname,varname1,varname2,varname3,varname4,varname5
  character(len=20) :: dname1,dname2,dname3
  character(len=20) :: numsfstr
  integer :: n,nobs,ierr,nmod
  real, dimension(neig) :: dimeig
  real, dimension(neigfin) :: dimeigfin
  real, dimension(neigfinaw) :: dimeigfinaw
  real, dimension(maxobs,nemsteps*nbox+nbkgnd) :: knest_trim
  real, dimension(nemsteps*nbox+nbkgnd,nemsteps*nbox+nbkgnd) :: cova
  real(kind=8), dimension(nbox*nemsteps+nbkgnd,nobs) :: gain
  real(kind=8), dimension(nobs,nobs) :: covagg
  real, dimension(nobs,nobs) :: covr
  real, dimension(nbox*nemsteps+nbkgnd) :: x,xb
  real, dimension(maxobs) :: yobs,ymod,ybgi,yunc
  real, dimension(nbkgnd) :: ebgb,ebga
  real, dimension(nemsteps) :: timeout
  character(len=7), dimension(nobsteps*numrec) :: yrec
  integer, dimension(maxobs) :: ydat
  real, dimension(nobs) :: ypos,ybgpos
  real(kind=8), dimension(maxrows,8) :: modout
 
 
  ! numeric scaling factor as string for file attributes
  write(numsfstr,'(ES12.1)') numscale

  ! coarse grid definition (nbox_xy)
  fileout='nbox_xy.nc'
  varname='nbox_xy'
  varunit='none'
  dname1='dimx'
  dname2='dimy'
  comment='inversion grid definition'
  call write2Dncdf(dirinvert,fileout,varname,varunit,real(nbox_xy),&
                   &numxnest,numynest,dimxnest,dimynest,dname1,dname2,comment)

  ! grid operator (gridoper)

  fileout='grid_operator.nc'
  varname='gridoper'
  varunit='none'
  dname1='dimx'
  dname2='dimy'
  comment='inversion grid definition'
  call write2Dncdf(dirinvert,fileout,varname,varunit,real(gridoper),&
                   &nbox,numxgrid*numygrid,dimbox,dimxy,dname1,dname2,comment)

  ! grid coarse operator (gridcrs)

  if(calcbg) then
    fileout='grid_coarse.nc'
    varname='gridcrs'
    varunit='none'
    dname1='dimx'
    dname2='dimy'
    comment='inversion grid definition'
    call write2Dncdf(dirinvert,fileout,varname,varunit,real(gridcrs),&
                     &ncrs,numxgrid*numygrid,dimcrs,dimxy,dname1,dname2,comment)
  endif

  ! kernel matrix trimmed to observations (knest_trim)

  fileout='knest_trim.nc'
  varname='kernel'
  varunit='sm3/kg'
  dname1='dimr'
  dname2='dimbx'
  comment='surface influence kernel matrix'
  ! scale by numscale
  knest_trim(:,1:nbox*nemsteps)=knest_trim(:,1:nbox*nemsteps)*numscale
  call write2Dncdf(dirinvert,fileout,varname,varunit,knest_trim(1:nobs,:),&
                  &nobs,nbox*nemsteps+nbkgnd,dimr,dimbx,dname1,dname2,comment)
  ! undo scaling by numscale 
  knest_trim(:,1:nbox*nemsteps)=knest_trim(:,1:nbox*nemsteps)/numscale  

  ! kernel matrix at fine spatial resolution and averaged to observation time resolution (knest_fintrim)

  if(.not.lowmemory) then
    fileout='knest_fintrim.nc'
    varname='kernel'
    varunit='sm3/kg'
    dname1='dimr'
    dname2='dimb'
    comment='surface influence kernel matrix'
    call write2Dncdf(dirinvert,fileout,varname,varunit,knest_fintrim*numscale,&
                    &maxobs,numxnest*numynest,dimr,dimxynest,dname1,dname2,comment)
  endif

  ! prior error covariance matrix eigen-vectors and values at coarse resolution (covb)

  fileout='covb.nc'
  varname='evectors'
  varname1='evalues'
  varunit='none'
  dname1='dim1'
  dname2='dim2'
  do n=1,neig
    dimeig(n)=n
  end do
  comment='eigenvectors of the prior error covariance matrix scaled by '//trim(numsfstr)
  call writeeigncdf(dirinvert,fileout,varname,varname1,varunit,real(bevec,kind=4),real(beval,kind=4),&
                  &nbox,neig,dimbox,dimeig,dname1,dname2,comment)

  ! prior error covariance matrix eigen-vectors and values at fine resolution (covbfin)

  if(.not.lowmemory) then
    fileout='covbfin.nc'
    varname='evectors'
    varname1='evalues'
    varunit='none'
    dname1='dim1'
    dname2='dim2'
    do n=1,neigfin
      dimeigfin(n)=n
    end do
    comment='eigenvectors of prior error covariance matrix scaled by '//trim(numsfstr)
    call writeeigncdf(dirinvert,fileout,varname,varname1,varunit,real(bfinevec,kind=4),real(bfineval,kind=4),&
                    &numxnest*numynest,neigfin,dimxynest,dimeigfin,dname1,dname2,comment)
  endif

  ! prior error covariance matrix eigen-vectors and values at fine resolution (covbfinaw)

  if(.not.lowmemory.and.scaleflx.eq.2) then
    fileout='covbfinaw.nc'
    varname='evectors'
    varname1='evalues'
    varunit='none'
    dname1='dim1'
    dname2='dim2'
    do n=1,neigfinaw
      dimeigfinaw(n)=n
    end do
    comment='eigenvectors of area weighted prior error covariance matrix scaled by '//trim(numsfstr)
    call writeeigncdf(dirinvert,fileout,varname,varname1,varunit,real(bfinawevec,kind=4),real(bfinaweval,kind=4),&
                    &numxnest*numynest,neigfinaw,dimxynest,dimeigfinaw,dname1,dname2,comment)
  endif

  ! observation error covariance matrix (covr)

  fileout='covr.nc'
  varname='covr'
  varunit='vmr^2'
  dname1='dim1'
  dname2='dim2'
  comment='observation error covariance matrix'
  call write2Dncdf(dirinvert,fileout,varname,varunit,covr,&
                  &nobs,nobs,dimr,dimr,dname1,dname2,comment)

  ! aggregation error covariance matrix (covagg)

  if(.not.lowmemory) then
    fileout='covagg.nc'
    varname='covagg'
    varunit='vmr^2'
    dname1='dim1'
    dname2='dim2'
    comment='aggregation error covariance matrix'
    call write2Dncdf(dirinvert,fileout,varname,varunit,real(covagg,kind=4),&
                    &nobs,nobs,dimr,dimr,dname1,dname2,comment)
  endif

  ! prior state vector at coarse resolution (xb)

  open(100,file=trim(dirinvert)//'prior.txt',status='replace',action='write',iostat=ierr)
  do n=1,nemsteps*nbox
    write(100,*) xb(n)/numscale
  end do
  do n=1,nbkgnd
    write(100,*) xb(nbox*nemsteps+n)
  end do
  close(100)

  ! areas of gridcells in coarse grid (area_box)

  open(100,file=trim(dirinvert)//'area_box.txt',status='replace',action='write',iostat=ierr)
  do n=1,nbox
    write(100,*) area_box(n)
  end do
  close(100)

  ! simulated and background mixing ratios a posteriori

  ypos(:)=0.
  ybgpos(:)=0.
  do n=1,nobs
    ypos(n)=dot_product(knest_trim(n,:),x)
    if(calcbg.and.optimizbg) then
      ybgpos(n)=dot_product(knest_trim(n,nbox*nemsteps+1:nbox*nemsteps+nbkgnd),&
                            &x(nbox*nemsteps+1:nbox*nemsteps+nbkgnd))
    endif
  end do

  ymod(:)=0.
  do n=1,nobs
    ymod(n)=dot_product(knest_trim(n,:),xb)
  end do

  ! observations and simulations a priori and a posteriori

  open(100,file=trim(dirinvert)//'monitor.txt',status='replace',action='write',iostat=ierr)
  do n=1,nobs
    write(100,fmt='(A7,1X,I10,1X,F9.3,1X,F11.3,1X,F11.3,1X,F9.3,1X,F9.3,1X,F9.3)') &
          &yrec(n),ydat(n),yobs(n),ymod(n),ypos(n),ybgi(n),ybgpos(n),yunc(n)
  end do
  close(100)

  ! posterior state vector on coarse grid (x)

  open(100,file=trim(dirinvert)//'posterior.txt',status='replace',action='write',iostat=ierr)
  do n=1,nemsteps*nbox
    write(100,*) x(n)/numscale
  end do
  do n=1,nbkgnd
    write(100,*) x(nbox*nemsteps+n)
  end do
  close(100)

  ! prior and posterior background scalars and scalar errors

  if(calcbg.and.optimizbg) then
    open(100,file=trim(dirinvert)//'bgscalars.txt',status='replace',action='write',iostat=ierr)
    do n=1,nbkgnd
       write(100,fmt='(F6.4,1X,F6.4,1X,F6.4,1X,F6.4)') &
             &xb(nbox*nemsteps+n),x(nbox*nemsteps+n),ebgb(n),ebga(n)
    end do
    close(100)
  endif

  ! posterior error covariance matrix (cova)

  fileout='cova.nc'
  varname='cova'
  varunit='(kg/m3/s)^2'
  dname1='dim1'
  dname2='dim2'
  comment='posterior error covariance matrix, errors scaled by '//trim(numsfstr)
  call write2Dncdf(dirinvert,fileout,varname,varunit,cova,&
                   &nbox*nemsteps+nbkgnd,nbox*nemsteps+nbkgnd,dimbx,dimbx,&
                   &dname1,dname2,comment)

  ! gain matrix (gain)

  fileout='gain.nc'
  varname='gain'
  varunit='none'
  dname1='dim1'
  dname2='dim2'
  comment='gain matrix of contribution functions'
  call write2Dncdf(dirinvert,fileout,varname,varunit,real(gain,kind=4),&
                   &nbox*nemsteps+nbkgnd,nobs,dimbx,dimr,&
                   &dname1,dname2,comment)

  ! posterior simulated mixing ratios at all timesteps (modout)
  ! columns:
  ! 1. receptor name
  ! 2. modelled concentration number (for moving receptors this is the same as the observation number)
  ! 3. date (yyyymmdd)
  ! 4. model timestep (hours)
  ! 5. prior modelled concentration
  ! 6. posterior modelled concentration
  ! 7. prior modelled background (if calcbg is false this is zero)
  ! 8. posterior modelled background (if calcbg or optimizbg is false this is zero)
  open(100,file=trim(dirinvert)//'modout.txt',status='replace',action='write',iostat=ierr)
  do n=1,nmod
    write(100,fmt='(A7,1X,I5,1X,I8,1X,I5,1X,F9.3,1X,F9.3,1X,F9.3,1X,F9.3)') &
          &recname(int(modout(n,1))),int(modout(n,2)),int(modout(n,3)),int(modout(n,4)),&
          &modout(n,5),modout(n,6),modout(n,7),modout(n,8)
  end do
  close(100)

  ! prior and posterior fluxes and errors at fine resolution

  fileout='emisflex.nc'
  varname='emis_flex'
  varunit='kg/m2/s'
  dname1='longitude'
  dname2='latitude'
  dname3='time'
  comment='prior emissions regridded to FLEXPART resolution'
  call write3Dncdf(dirinvert,fileout,varname,varunit,emisflex,&
                   &numxgrid,numygrid,ntemi,gridlon,gridlat,timemi,&
                   &dname1,dname2,dname3,comment)

  fileout='analysis.nc'
  varname='emis_post'
  varunit='kg/m2/s'
  dname1='longitude'
  dname2='latitude'
  dname3='time'
  comment='posterior emissions on fine grid'
  call write3Dncdf(dirinvert,fileout,varname,varunit,emispost,&
                   &numxnest,numynest,nemsteps,nestlon,nestlat,timeout,&
                   &dname1,dname2,dname3,comment)

  varname='emis_prior'
  varunit='kg/m2/s'
  comment='prior emissions on fine grid'
  call add3Dncdf(dirinvert,fileout,varname,varunit,emisprior(ixl:ixr,iyl:iyu,:),&
                 &numxnest,numynest,nemsteps,nestlon,nestlat,timeout,&
                 &dname1,dname2,dname3,comment)

  varname='error_post'
  varunit='kg/m2/s'
  comment='posterior errors on fine grid'
  call add3Dncdf(dirinvert,fileout,varname,varunit,errpost,&
                 &numxnest,numynest,nemsteps,nestlon,nestlat,timeout,&
                 &dname1,dname2,dname3,comment)

  varname='error_prior'
  varunit='kg/m2/s'
  comment='prior errors on fine grid'
  call add3Dncdf(dirinvert,fileout,varname,varunit,errpri,&
                 &numxnest,numynest,nemsteps,nestlon,nestlat,timeout,&
                 &dname1,dname2,dname3,comment)

end subroutine save


