!---------------------------------------------------------------------------------------
! FLEXINVERT : savetransport
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Saves transport related files so that these can be used in a subsequent run
!! using the restart mode.
!! The files saved are: 
!!  - nbox_xy.nc
!!  - grid_operator.nc
!!  - grid_coarse.nc
!!  - knest.nc
!!  - kout.nc (only if calcbg is true)
!!  - knest_fin.nc (only if lowmemory is false)
!---------------------------------------------------------------------------------------

subroutine savetransport(knest,kout)

  use mod_var
  
  implicit none

  character(len=100) :: fileout,varunit,comment,varname
  character(len=20) :: dname1,dname2
  real, dimension(maxrows,nemsteps*nbox) :: knest
  real, dimension(maxrows,ncrs) :: kout
  integer, dimension(maxrows) :: dimrr
  integer, dimension(nbox*nemsteps) :: dimbb
  integer, dimension(numxnest*numynest) :: dimbf
  integer :: i

  do i=1,maxrows
    dimrr(i)=i
  end do

  do i=1,nbox*nemsteps
    dimbb(i)=i
  enddo

  if(.not.lowmemory) then
    do i=1,numxnest*numynest
      dimbf(i)=i
    enddo
  endif

  ! coarse grid definition (nbox_xy)

  fileout='nbox_xy.nc'
  varname='nbox_xy'
  varunit='none'
  dname1='dimx'
  dname2='dimy'
  comment='inversion grid definition'
  call write2Dncdf(dirinvert,fileout,varname,varunit,real(nbox_xy),&
                   &numxnest,numynest,dimxnest,dimynest,dname1,dname2,comment)

  ! grid operator (gridoper)

  fileout='grid_operator.nc'
  varname='gridoper'
  varunit='none'
  dname1='dimx'
  dname2='dimy'
  comment='inversion grid definition'
  call write2Dncdf(dirinvert,fileout,varname,varunit,real(gridoper),&
                   &nbox,numxgrid*numygrid,dimbox,dimxy,dname1,dname2,comment)

  ! grid coarse operator (gridcrs)

  if(calcbg) then
    fileout='grid_coarse.nc'
    varname='gridcrs'
    varunit='none'
    dname1='dimx'
    dname2='dimy'
    comment='inversion grid definition'
    call write2Dncdf(dirinvert,fileout,varname,varunit,real(gridcrs),&
                     &ncrs,numxgrid*numygrid,dimcrs,dimxy,dname1,dname2,comment)
  endif

  ! kernel matrix for nested domain (knest)

  fileout='knest.nc'
  varname='knest'
  varunit='sm3/kg'
  dname1='dimr'
  dname2='dimbx'
  comment='surface influence kernel matrix'
  call write2Dncdf(dirinvert,fileout,varname,varunit,knest,&
                  &maxrows,nbox*nemsteps,dimrr,dimbb,dname1,dname2,comment)

  ! kernel matrix for nested domain at fine resolution (knest_fin) 

  if(.not.lowmemory) then
    fileout='knest_fin.nc'
    varname='knest_fin'
    varunit='sm3/kg'
    dname1='dimr'
    dname2='dimb'
    comment='surface influence kernel matrix'
    call write2Dncdf(dirinvert,fileout,varname,varunit,knest_fin,&
                    &maxrows,numxnest*numynest,dimrr,dimbf,dname1,dname2,comment)
  endif

  ! kernel matrix for the outside domain (kout)

  if(calcbg) then
    fileout='kout.nc'
    varname='kout'
    varunit='sm3/kg'
    dname1='dimr'
    dname2='dimcrs'
    comment='surface influence kernel matrix'
    call write2Dncdf(dirinvert,fileout,varname,varunit,kout,&
                    &maxrows,ncrs,dimrr,dimcrs,dname1,dname2,comment)
  endif


end subroutine savetransport


