!---------------------------------------------------------------------------------------
! FLEXINVERT : timeseries
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Read gridded flexpart output files to calculate surface influence
!! and multiply with the posterior emissions to simulate concentrations from
!! full spatially and temporally resolved data
!---------------------------------------------------------------------------------------

subroutine timeseries(bkgnd,kout,time,modout,nmod,xb,x)

  use mod_var
  use mod_dates

  implicit none

  character(len=100) :: dirflexrec,datestr,filename
  integer :: k,n,nr,nt,ii,i1,i2,ix,jy,kz,it,cnt,ind,ierr
  integer :: yyyymmddhh,preyyyymmddhh,yyyymm,preyyyymm,yyyymmdd,yyyy,mm,dd,hh
  integer :: numpoint,timestep,rnum,nmod,eomday
  real(kind=8) :: jd,jdi,jtime
  real :: pri,pos
  real, dimension(maxrows,ncrs) :: bkgnd
  real, dimension(maxrows,ncrs) :: kout
  real, dimension(nemsteps) :: time
  real, dimension(nzmax) :: outheight
  integer, dimension(maxpoint) :: releases
  real, dimension(nbox*nemsteps+nbkgnd) :: xb,x
  real, dimension(numxgrid,numygrid,numzlev,maxpoint) :: grid,mgrid
  real(kind=8), dimension(maxrows,8) :: modout

  modout(:,:)=0.

  ind=0
  do nr=1,numrec
    rnum=0
    jd=juldatei
    jdi=juldatei
    do while (jd.le.juldatef)
      yyyymmddhh=caldate(jd)
      yyyymm=yyyymmddhh/10000
      eomday=calceomday(yyyymm)
      preyyyymm=yyyymm
      preyyyymmddhh=yyyymmddhh
      write(datestr,'(i6)') yyyymm
      dirflexrec=trim(dirflex)//trim(recname(nr))//'/'//trim(datestr)//'/'
      ! adjust start date for trajectories going back in time
      jd=jd-trajlen
      filename=trim(dirflexrec)//'header_grid_time'
!      filename=trim(dirflexrec)//'header'
      call readheader(filename,numpoint,outheight,releases)
      ! read all grids for trajectories in this directory
      mgrid(:,:,:,:)=0.
      cnt=0
      do while(yyyymm.le.preyyyymm)
        call readgrid(dirflexrec,jd,numpoint,grid)
        mgrid=mgrid+grid
        jd=jd+float(abs(loutstep))/3600./24.
        if(jd.gt.juldatef) go to 10
        yyyymmddhh=caldate(jd)
        yyyymmdd=yyyymmddhh/100
        yyyymm=yyyymmddhh/10000
        cnt=cnt+1
      end do
10    continue
      ! convert from ppt mass mixing ratio to volume mixing ratio
      mgrid=mgrid*coef*28.97/molarmass
      do nt=1,numpoint
        pri=0.
        pos=0.
        ind=ind+1
        rnum=rnum+1
        ! hour of trajectory from start of month 
        ! note: releases gives hour since 1st release in backwards time
        timestep=int((jdi-juldatei)*24.)+eomday*24+releases(nt)/3600
        ! calculate time index to emissions
        yyyymmddhh=caldate(jdi+float(eomday*24+releases(nt)/3600)/24.)
        yyyymmdd=yyyymmddhh/100
        yyyy=yyyymmddhh/1000000
        mm=(yyyymmddhh-yyyy*1000000)/10000
        dd=(yyyymmddhh-yyyy*1000000-mm*10000)/100
        hh=yyyymmddhh-yyyy*1000000-mm*10000-dd*100
        jtime=float(yyyy)+(float(mm)-1.)/12.+(float(dd)-1.)/365.+float(hh)/365./24.
        it=minloc(abs(time(:)-jtime),dim=1,mask=abs(time(:)-jtime).eq.minval(abs(time(:)-jtime)))
        do jy=1,numynest
          do ix=1,numxnest
            if(isnan(emisprior(ixl+ix-1,iyl+jy-1,it)).or.isnan(emispost(ix,jy,it))) go to 20
            pri=pri+mgrid(ixl+ix-1,iyl+jy-1,1,nt)*emisprior(ixl+ix-1,iyl+jy-1,it)/outheight(1)
            pos=pos+mgrid(ixl+ix-1,iyl+jy-1,1,nt)*emispost(ix,jy,it)/outheight(1)
20          continue
          end do
        end do
        modout(ind,1)=real(nr,kind=8)
        modout(ind,2)=real(rnum,kind=8)
        modout(ind,3)=real(yyyymmdd,kind=8)
        modout(ind,4)=real(timestep,kind=8)
        modout(ind,5)=real(pri,kind=8)
        modout(ind,6)=real(pos,kind=8)
        if(calcbg) then
          if(optimizbg) then 
            ! using background optimization
            modout(ind,7)=real(dot_product((bkgnd(ind,:)+kout(ind,:)),xb(nbox*nemsteps+1:nbox*nemsteps+nbkgnd)),kind=8)
            modout(ind,8)=real(dot_product((bkgnd(ind,:)+kout(ind,:)),x(nbox*nemsteps+1:nbox*nemsteps+nbkgnd)),kind=8)
          else
            ! no background optimization
            modout(ind,7)=real(sum(bkgnd(ind,:))+sum(kout(ind,:)),kind=8)
            modout(ind,8)=real(sum(bkgnd(ind,:))+sum(kout(ind,:)),kind=8)
          endif ! optimizbg
        else
          ! assumes observations are departures from background   
          modout(ind,7)=0.
          modout(ind,8)=0.
        endif ! calcbg
      end do ! nt
      jdi=jdi+float(eomday)
    end do ! jd
  end do ! numrec

  nmod=ind

end subroutine timeseries


