!---------------------------------------------------------------------------------------
! FLEXINVERT : writeeigncdf 
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Writes a 2D array and 1D array to a netcdf file
!---------------------------------------------------------------------------------------

subroutine writeeigncdf(dirout,fileout,varname2d,varname1d,varunit,var2d,var1d,&
                      &nd1,nd2,dim1,dim2,dname1,dname2,comment)

  use mod_var

  implicit none

  include 'netcdf.inc'

  character(len=100) :: dirout,fileout,varname2d,varname1d,varunit,comment
  character(len=20) :: dname1,dname2
  integer :: nd1,nd2,did1,did2
  real :: dim1(nd1),dim2(nd2)
  integer :: nid,varid,varid2d,varid1d,ierr,ind
  integer :: dimids(2)
  real :: var2d(nd1,nd2)
  real :: var1d(nd2)

  ! open netcdf file
  write(logid,*) 'Writing file: '//trim(dirout)//trim(fileout)
  ierr=nf_create(trim(dirout)//trim(fileout),NF_CLOBBER,nid)
  if(ierr.ne.0) then
    write(logid,*) 'Error creating file: ',trim(dirout)//trim(fileout)
  endif

  ! define dimensions
  ierr=nf_def_dim(nid,dname1,nd1,did1)
  ierr=nf_def_dim(nid,dname2,nd2,did2)
  ierr=nf_enddef(nid)

  ! define dimension variables
  ierr=nf_redef(nid)
  ierr=nf_def_var(nid,dname1,NF_FLOAT,1,did1,varid)
  ind=len_trim(dname1)
  ierr=nf_put_att_text(nid,varid,'title',len_trim(dname1),dname1(1:ind))
  ierr=nf_enddef(nid)
  ierr=nf_put_var_real(nid,varid,dim1)
  ierr=nf_redef(nid)
  ind=len_trim(dname2)
  ierr=nf_def_var(nid,dname2,NF_FLOAT,1,did2,varid)
  ierr=nf_put_att_text(nid,varid,'title',len_trim(dname2),dname2(1:ind))
  ierr=nf_enddef(nid)
  ierr=nf_put_var_real(nid,varid,dim2)

  ! define dependent variable: var2D
  dimids(:)=(/did1,did2/)
  ierr=nf_redef(nid)
  ierr=nf_def_var(nid,varname2d,NF_FLOAT,2,dimids,varid2d)
  ind=len_trim(varname2d)
  ierr=nf_put_att_text(nid,varid2d,'title',len_trim(varname2d),varname2d(1:ind))
  ind=len_trim(varunit)
  ierr=nf_put_att_text(nid,varid2d,'unit',len_trim(varunit),varunit(1:ind))
  ierr=nf_enddef(nid)

  ! define dependent variable: var1D
  ierr=nf_redef(nid)
  ierr=nf_def_var(nid,varname1d,NF_FLOAT,1,did2,varid1d)
  ind=len_trim(varname1d)
  ierr=nf_put_att_text(nid,varid1d,'title',len_trim(varname1d),varname1d(1:ind))
  ind=len_trim(varunit)
  ierr=nf_put_att_text(nid,varid1d,'unit',len_trim(varunit),varunit(1:ind))
  ierr=nf_enddef(nid)

  ! insert 2D variable
  ierr=nf_put_var_real(nid,varid2d,var2d)
  if(ierr.ne.0) then
    write(logid,*) 'Error writing variable '//trim(varname2d) 
  endif

  ! insert 1D variable
  ierr=nf_put_var_real(nid,varid1d,var1d)
  if(ierr.ne.0) then
    write(logid,*) 'Error writing variable '//trim(varname1d)
  endif

  ! insert variable attribute
  ierr=nf_redef(nid)
  ind=len_trim(comment)
  ierr=nf_put_att_text(nid,varid,'comment',len_trim(comment),comment(1:ind))
  ierr=nf_enddef(nid)

  ierr=nf_close(nid)

end subroutine writeeigncdf

